@@ -1,14 +1,27 @@
 /*
-  $Header: /cvs/src/chrony/conf.h,v 1.16 1999/04/29 20:29:34 richard Exp $
+  $Header: /cvs/src/chrony/conf.h,v 1.25 2003/09/22 21:22:30 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2003
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -18,12 +31,16 @@
 #ifndef GOT_CONF_H
 #define GOT_CONF_H
 
+extern char *CNF_GetRtcDevice(void);
+
 extern void CNF_ReadFile(const char *filename);
 
 extern void CNF_AddSources(void);
+extern void CNF_AddBroadcasts(void);
 
 extern void CNF_ProcessInitStepSlew(void (*after_hook)(void *), void *anything);
 
+extern unsigned short CNF_GetAcquisitionPort(void);
 extern unsigned short CNF_GetNTPPort(void);
 extern char *CNF_GetDriftFile(void);
 extern char *CNF_GetLogDir(void);
@@ -43,6 +60,10 @@ extern void CNF_GetLogChange(int *enabled, double *threshold);
 extern void CNF_GetMailOnChange(int *enabled, double *threshold, char **user);
 extern int CNF_GetNoClientLog(void);
 extern void CNF_GetBindAddress(unsigned long *addr);
+extern void CNF_GetBindCommandAddress(unsigned long *addr);
+extern char *CNF_GetPidFile(void);
+extern void CNF_GetLinuxHz(int *set, int *hz);
+extern void CNF_GetLinuxFreqScale(int *set, double *freq_scale);
 
 /* Value returned in ppm, as read from file */
 extern double CNF_GetMaxUpdateSkew(void);
