@@ -1,14 +1,27 @@
 /*
-  $Header: /cvs/src/chrony/logging.h,v 1.13 1999/04/19 20:27:29 richard Exp $
+  $Header: /cvs/src/chrony/logging.h,v 1.15 2002/02/28 23:27:10 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2002
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -64,7 +77,7 @@ extern void LOG_Finalise(void);
 extern void LOG_Line_Function(LOG_Severity severity, LOG_Facility facility, const char *format, ...);
 
 /* Logging function for fatal errors */
-extern volatile void LOG_Fatal_Function(LOG_Facility facility, const char *format, ...);
+extern void LOG_Fatal_Function(LOG_Facility facility, const char *format, ...);
 
 /* Position in code reporting function */
 extern void LOG_Position(const char *filename, int line_number, const char *function_name);
