@@ -43,32 +43,6 @@
 #include <sys/time.h>
 #include <sys/types.h>
 
-#ifdef HAS_SPINLOCK_H
-#include <linux/spinlock.h>
-#else
-/* Include dummy definition of spinlock_t to cope with earlier kernels. */
-typedef int spinlock_t;
-#endif
-
-/* This is a complete hack since the alpha sys/io.h needs these types
- * but does not arrange them to be defined.  This is almost certainly
- * not how one should do these things.  -- broonie
- */
-#include <linux/types.h>
-#ifdef __alpha__
-typedef __u8  u8;
-typedef __u16 u16;
-typedef __u32 u32;
-typedef __u64 u64;
-#endif
-
-#if defined(__i386__) /* || defined(__sparc__) */
-#include <linux/mc146818rtc.h>
-#else
-#include <linux/rtc.h>
-#define RTC_UIE 0x10		/* update-finished interrupt enable */
-#endif
-
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -84,10 +58,23 @@ typedef __u64 u64;
 #include "regress.h"
 #include "rtc.h"
 #include "rtc_linux.h"
+#include "io_linux.h"
 #include "conf.h"
 #include "memory.h"
 #include "mkdirpp.h"
 
+struct rtc_time {
+	int tm_sec;
+	int tm_min;
+	int tm_hour;
+	int tm_mday;
+	int tm_mon;
+	int tm_year;
+	int tm_wday;
+	int tm_yday;
+	int tm_isdst;
+};
+
 /* ================================================== */
 /* Forward prototypes */
 
@@ -187,7 +174,7 @@ static double file_ref_offset, file_rate_ppm;
 /* ================================================== */
 
 /* Flag to remember whether to assume the RTC is running on UTC */
-static int rtc_on_utc = 0;
+static int rtc_on_utc = 1;
 
 /* ================================================== */
 
@@ -239,15 +226,18 @@ accumulate_sample(time_t rtc, struct timeval *sys)
     discard_samples(NEW_FIRST_WHEN_FULL);
   }
 
-  rtc_sec[n_samples] = rtc;
 
   /* Always use most recent sample as reference */
+  /* use sample only if n_sample is not negative*/
+  if(n_samples >=0)
+  {
   rtc_ref = rtc;
-
+  rtc_sec[n_samples] = rtc;
   rtc_trim[n_samples] = 0.0;
   system_times[n_samples] = *sys;
-  ++n_samples;
   ++n_samples_since_regression;
+  }
+  ++n_samples;
   return;
   
 }
@@ -755,7 +745,11 @@ handle_initial_trim(void)
   run_regression(1, &coefs_valid, &coef_ref_time, &coef_seconds_fast, &coef_gain_rate);
 
   n_samples_since_regression = 0;
-  n_samples = 0;
+
+  /* Set sample number to -1 so the next sample is not used, as it will not yet be corrected for System Trim*/
+
+  n_samples = -1;
+
 
   read_coefs_from_file();
 
@@ -879,10 +873,22 @@ read_from_device(void *any)
   int error = 0;
 
   status = read(fd, &data, sizeof(data));
+  if (operating_mode == OM_NORMAL)
+      status = read(fd, &data, sizeof(data));
   if (status < 0) {
+    /* This looks like a bad error : the file descriptor was indicating it was
+     * ready to read but we couldn't read anything.  Give up. */
     LOG(LOGS_ERR, LOGF_RtcLinux, "Could not read flags %s : %s", CNF_GetRtcDevice(), strerror(errno));
     error = 1;
-    goto turn_off_interrupt;
+    SCH_RemoveInputFileHandler(fd);
+    switch_interrupts(0); /* Likely to raise error too, but just to be sure... */
+    close(fd);
+    fd = -1;
+    if (logfile) {
+      fclose(logfile);
+      logfile = NULL;
+    }
+    return;
   }    
 
   if ((data & RTC_UIE) == RTC_UIE) {
