@@ -1,14 +1,27 @@
 /*
-  $Header: /cvs/src/chrony/sysincl.h,v 1.3 1999/05/02 20:22:08 richard Exp $
+  $Header: /cvs/src/chrony/sysincl.h,v 1.11 2003/09/22 21:22:30 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2003
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -20,22 +33,25 @@
 #ifndef GOT_SYSINCL_H
 #define GOT_SYSINCL_H
 
-#if defined (SOLARIS) || defined(SUNOS) || defined(LINUX)
+#if defined (SOLARIS) || defined(SUNOS) || defined(LINUX) || defined(__NetBSD__)
 
+#if !defined(__NetBSD__) && !defined(__FreeBSD__)
 #include <alloca.h>
+#endif
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <float.h>
+#if !defined(__FreeBSD__)
 #include <malloc.h>
+#endif
 #include <math.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <signal.h>
 #include <stdarg.h>
 #include <stddef.h>
-#include <stddef.h> /* To get definition of offsetof */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -46,6 +62,19 @@
 #include <sys/types.h>
 #include <syslog.h>
 #include <time.h>
+
+#if HAS_STDINT_H
+#include <stdint.h>
+#elif defined(HAS_INTTYPES_H)
+#include <inttypes.h>
+#else
+/* Tough */
+#endif
+
+/* One or other of these to make getsid() visible */
+#define __EXTENSIONS__ 1
+#define __USE_XOPEN_EXTENDED 1
+
 #include <unistd.h>
 
 #endif
@@ -56,6 +85,10 @@
 #include <nlist.h>
 #endif
 
+#if defined (HAS_NO_BZERO)
+#define bzero(ptr,n) memset(ptr,0,n)
+#endif /* HAS_NO_BZERO */
+
 #if defined (WINNT)
 
 /* Designed to work with the GCC from the GNAT-3.10 for Win32
